<?php

declare(strict_types=1);

namespace Dedi\SyliusVoucherPlugin\Generator;

use Dedi\SyliusVoucherPlugin\Repository\VoucherRepositoryInterface;
use Exception;
use Safe\Exceptions\PcreException;
use function Safe\preg_replace;

final class VoucherCodeGenerator implements VoucherCodeGeneratorInterface
{
    /** @var VoucherRepositoryInterface */
    private $voucherRepository;

    /** @var int */
    private $codeLength;

    public function __construct(VoucherRepositoryInterface $voucherRepository, int $codeLength)
    {
        $this->voucherRepository = $voucherRepository;
        $this->codeLength = $codeLength;
    }

    /**
     * @throws Exception
     * @throws PcreException
     */
    public function generate(): string
    {
        do {
            // if we didn't remove the 'hard to read' characters we would only have to
            // generate codeLength / 2 bytes because hex uses two characters to represent one byte
            $code = bin2hex(random_bytes($this->codeLength));
            $code = preg_replace('/[01]/', '', $code); // remove hard to read characters
            $code = mb_strtoupper(mb_substr($code, 0, $this->codeLength));
        } while (mb_strlen($code) !== $this->codeLength || $this->exists($code));

        return $code;
    }

    private function exists(string $code): bool
    {
        return null !== $this->voucherRepository->findOneByCode($code);
    }
}
